package javax.management.remote;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import com.sun.jmx.remote.util.EnvHelp;
import java.io.IOException;
import java.net.MalformedURLException;
import javax.management.MBeanServer;
import java.util.Map;
import com.sun.jmx.remote.util.ClassLogger;

public class JMXConnectorServerFactory
{
    public static final String DEFAULT_CLASS_LOADER = "jmx.remote.default.class.loader";
    public static final String DEFAULT_CLASS_LOADER_NAME = "jmx.remote.default.class.loader.name";
    public static final String PROTOCOL_PROVIDER_PACKAGES = "jmx.remote.protocol.provider.pkgs";
    public static final String PROTOCOL_PROVIDER_CLASS_LOADER = "jmx.remote.protocol.provider.class.loader";
    private static final String PROTOCOL_PROVIDER_DEFAULT_PACKAGE = "com.sun.jmx.remote.protocol";
    private static final ClassLogger logger;
    
    private JMXConnectorServerFactory() {
    }
    
    private static JMXConnectorServer getConnectorServerAsService(ClassLoader classLoader, JMXServiceURL obj, Map<String, ?> map, MBeanServer mBeanServer) throws IOException {
        Iterator<JMXConnectorServerProvider> providerIterator = JMXConnectorFactory.getProviderIterator(JMXConnectorServerProvider.class, classLoader);
        IOException ex = null;
        while (providerIterator.hasNext()) {
            try {
                return ((JMXConnectorServerProvider)providerIterator.next()).newJMXConnectorServer(obj, map, mBeanServer);
            }
            catch (JMXProviderException ex2) {
                throw ex2;
            }
            catch (Exception ex3) {
                if (JMXConnectorServerFactory.logger.traceOn()) {
                    JMXConnectorServerFactory.logger.trace("getConnectorAsService", "URL[" + obj + "] Service provider exception: " + ex3);
                }
                if (!(ex3 instanceof MalformedURLException) && ex == null) {
                    if ((ex3 instanceof IOException)) {
                        ex = (IOException)ex3;
                    }
                    else {
                        ex = (IOException)EnvHelp.initCause(new IOException(ex3.getMessage()), ex3);
                    }
                }
                else {
                    continue;
                }
            }
        }
        if (ex == null) {
            return null;
        }
        throw ex;
    }
    
    public static JMXConnectorServer newJMXConnectorServer(JMXServiceURL jmxServiceURL, Map<String, ?> map, MBeanServer mBeanServer) throws IOException {
        HashMap m;
        if (map == null) {
            m = new HashMap();
        }
        else {
            EnvHelp.checkAttributes(map);
            m = new HashMap(map);
        }
        Class<JMXConnectorServerProvider> clazz = JMXConnectorServerProvider.class;
        ClassLoader resolveClassLoader = JMXConnectorFactory.resolveClassLoader(m);
        String protocol = jmxServiceURL.getProtocol();
        JMXConnectorServerProvider jmxConnectorServerProvider = (JMXConnectorServerProvider)JMXConnectorFactory.getProvider(jmxServiceURL, m, "ServerProvider", clazz, resolveClassLoader);
        IOException cause = null;
        if (jmxConnectorServerProvider == null) {
            if (resolveClassLoader != null) {
                try {
                    JMXConnectorServer connectorServerAsService = getConnectorServerAsService(resolveClassLoader, jmxServiceURL, m, mBeanServer);
                    if (connectorServerAsService != null) {
                        return connectorServerAsService;
                    }
                }
                catch (JMXProviderException ex) {
                    throw ex;
                }
                catch (IOException ex2) {
                    cause = ex2;
                }
            }
            jmxConnectorServerProvider = (JMXConnectorServerProvider)JMXConnectorFactory.getProvider(protocol, "com.sun.jmx.remote.protocol", JMXConnectorFactory.class.getClassLoader(), "ServerProvider", clazz);
        }
        if (jmxConnectorServerProvider == null) {
            MalformedURLException throwable = new MalformedURLException("Unsupported protocol: " + protocol);
            if (cause == null) {
                throw throwable;
            }
            throw (MalformedURLException)EnvHelp.initCause(throwable, cause);
        }
        else {
            return jmxConnectorServerProvider.newJMXConnectorServer(jmxServiceURL, Collections.unmodifiableMap(m), mBeanServer);
        }
    }
    
    static {
        logger = new ClassLogger("javax.management.remote.misc", "JMXConnectorServerFactory");
    }
}
